==> get-2.12.0-beta.tar.gz: lib/src/instance/get_instance.dart <==
import 'package:get/get.dart';

class GetInstance {
  static GetInstance _get;
  factory GetInstance() {
    if (_get == null) _get = GetInstance._();
    return _get;
  }
  GetInstance._();

  Map<dynamic, dynamic> singl = Map();
  Map<dynamic, FcBuilderFunc> factory = Map();
  Map<String, String> routesKey = Map();
  static SmartManagement smartManagement = SmartManagement.full;
  static bool logEnable = true;

  void lazyPut<S>(FcBuilderFunc builder, {String tag}) {
    factory.putIfAbsent(getKey(S, tag), () => builder);
  }

  Future<S> putAsync<S>(FcBuilderFuncAsync<S> builder, {String tag}) async {
    // WidgetsFlutterBinding.ensureInitialized();
    return put<S>(await builder(), tag: tag);
  }

  /// Inject class on Get Instance Manager
  S put<S>(
    S dependency, {
    String tag,
    bool permanent = false,
    bool overrideAbstract = false,
    FcBuilderFunc<S> builder,
  }) {
    _insert(
        isSingleton: true,
        replace: overrideAbstract,
        //?? (("$S" == "${dependency.runtimeType}") == false),
        name: tag,
        permanent: permanent,
        builder: builder ?? (() => dependency));
    return find<S>(tag: tag);
  }

  /// Create a new instance from builder class
  /// Example
  /// Get.create(() => Repl());
  /// Repl a = Get.find();
  /// Repl b = Get.find();
  /// print(a==b); (false)
  void create<S>(
    FcBuilderFunc<S> builder, {
    String name,
  }) {
    _insert(isSingleton: false, name: name, builder: builder);
  }

  void _insert<S>({
    bool isSingleton,
    String name,
    bool replace = true,
    bool permanent = false,
    FcBuilderFunc<S> builder,
  }) {
    assert(builder != null);
    String key = getKey(S, name);
    if (replace) {
      singl[key] = FcBuilder<S>(isSingleton, builder, permanent);
    } else {
      singl.putIfAbsent(
          key, () => FcBuilder<S>(isSingleton, builder, permanent));
    }
  }

  void removeDependencyByRoute(String routeName) async {
    List<String> keysToRemove = [];
    routesKey.forEach((key, value) {
      // if (value == routeName && value != null) {
      if (value == routeName) {
        keysToRemove.add(key);
      }
    });
    keysToRemove.forEach((element) async {
      await delete(key: element);
    });
    keysToRemove.forEach((element) {
      routesKey?.remove(element);
    });
    keysToRemove.clear();
  }

  bool isRouteDependecyNull<S>({String name}) {
    return (routesKey[getKey(S, name)] == null);
  }

  bool isDependencyInit<S>({String name}) {
    String key = getKey(S, name);
    return routesKey.containsKey(key);
  }

  void registerRouteInstance<S>({String tag}) {
    //  print("Register route [$S] as ${Get.currentRoute}");
    routesKey.putIfAbsent(getKey(S, tag), () => Get.currentRoute);
  }

  S findByType<S>(Type type, {String tag}) {
    String key = getKey(type, tag);
    return singl[key].getSependency();
  }

  void initController<S>({String tag}) {
    String key = getKey(S, tag);
    final i = singl[key].getSependency();

    if (i is DisposableInterface) {
      i.onInit();
      if (GetInstance.logEnable) print('[GET] $key has been initialized');
    }
  }

  /// Find a instance from required class
  S find<S>({String tag, FcBuilderFunc<S> instance}) {
    String key = getKey(S, tag);
    bool callInit = false;
    if (isRegistred<S>(tag: tag)) {
      if (!isDependencyInit<S>() &&
          GetInstance.smartManagement != SmartManagement.onlyBuilder) {
        registerRouteInstance<S>(tag: tag);
        callInit = true;
      }

      FcBuilder builder = singl[key];
      if (builder == null) {
        if (tag == null) {
          throw "class ${S.toString()} is not register";
        } else {
          throw "class ${S.toString()} with tag '$tag' is not register";
        }
      }
      if (callInit) {
        initController<S>(tag: tag);
      }

      return singl[key].getSependency();
    } else {
      if (!factory.containsKey(key))
        throw " $S not found. You need call Get.put<$S>($S()) before";

      if (GetInstance.logEnable)
        print('[GET] $S instance was created at that time');
      S _value = put<S>(factory[key].call() as S);

      if (!isDependencyInit<S>() &&
          GetInstance.smartManagement != SmartManagement.onlyBuilder) {
        registerRouteInstance<S>(tag: tag);
        callInit = true;
      }

      if (GetInstance.smartManagement != SmartManagement.keepFactory) {
        factory.remove(key);
      }

      if (callInit) {
        initController<S>(tag: tag);
      }
      return _value;
    }
  }

  /// Remove dependency of [S] on dependency abstraction. For concrete class use Get.delete
  void remove<S>({String tag}) {
    String key = getKey(S, tag);
    FcBuilder builder = singl[key];
    final i = builder.dependency;

    if (i is DisposableInterface || i is GetController) {
      i.onClose();
      if (GetInstance.logEnable) print('[GET] onClose of $key called');
    }
    if (builder != null) builder.dependency = null;
    if (singl.containsKey(key)) {
      print('error on remove $key');
    } else {
      if (GetInstance.logEnable) print('[GET] $key removed from memory');
    }
  }

  String getKey(Type type, String name) {
    return name == null ? type.toString() : type.toString() + name;
  }

  bool reset({bool clearFactory = true, bool clearRouteBindings = true}) {
    if (clearFactory) factory.clear();
    if (clearRouteBindings) routesKey.clear();
    singl.clear();
    return true;
  }

  /// Delete class instance on [S] and clean memory
  Future<bool> delete<S>({String tag, String key}) async {
    String newKey;
    if (key == null) {
      newKey = getKey(S, tag);
    } else {
      newKey = key;
    }

    if (!singl.containsKey(newKey)) {
      print('Instance $newKey not found');
      return false;
    }

    FcBuilder builder = singl[newKey];
    if (builder.permanent) {
      (key == null)
          ? print(
              '[GET] [$newKey] has been marked as permanent, SmartManagement is not authorized to delete it.')
          : print(
              '[GET] [$newKey] has been marked as permanent, SmartManagement is not authorized to delete it.');
      return false;
    }
    final i = builder.dependency;

    if (i is DisposableInterface || i is GetController) {
      await i.onClose();
      if (GetInstance.logEnable) print('[GET] onClose of $newKey called');
    }

    singl.removeWhere((oldkey, value) => (oldkey == newKey));
    if (singl.containsKey(newKey)) {
      print('[GET] error on remove object $newKey');
    } else {
      if (GetInstance.logEnable) print('[GET] $newKey deleted from memory');
    }
    // Get().routesKey?.remove(key);
    return true;
  }

  /// check if instance is registred
  bool isRegistred<S>({String tag}) => singl.containsKey(getKey(S, tag));

  bool isPrepared<S>({String tag}) => factory.containsKey(getKey(S, tag));
}

typedef FcBuilderFunc<S> = S Function();

typedef FcBuilderFuncAsync<S> = Future<S> Function();

class FcBuilder<S> {
  bool isSingleton;
  FcBuilderFunc builderFunc;
  S dependency;
  bool permanent = false;

  FcBuilder(this.isSingleton, this.builderFunc, this.permanent);

  S getSependency() {
    if (isSingleton) {
      if (dependency == null) {
        dependency = builderFunc() as S;
      }
      return dependency;
    } else {
      return builderFunc() as S;
    }
  }
}

==> get-2.12.1-beta.tar.gz: lib/src/instance/get_instance.dart <==
import 'package:get/src/instance/typedefs.dart';
import 'package:get/src/root/get_config.dart';
import 'package:get/src/root/smart_management.dart';
import 'package:get/src/routes/routing.dart';
import 'package:get/src/rx/rx_interface.dart';

class GetInstance {
  static GetInstance _get;
  factory GetInstance() {
    if (_get == null) _get = GetInstance._();
    return _get;
  }
  GetInstance._();

  Map<dynamic, dynamic> singl = Map();
  Map<dynamic, FcBuilderFunc> factory = Map();
  Map<String, String> routesKey = Map();

  void lazyPut<S>(FcBuilderFunc builder, {String tag}) {
    factory.putIfAbsent(getKey(S, tag), () => builder);
  }

  Future<S> putAsync<S>(FcBuilderFuncAsync<S> builder, {String tag}) async {
    // WidgetsFlutterBinding.ensureInitialized();
    return put<S>(await builder(), tag: tag);
  }

  /// Inject class on Get Instance Manager
  S put<S>(
    S dependency, {
    String tag,
    bool permanent = false,
    bool overrideAbstract = false,
    FcBuilderFunc<S> builder,
  }) {
    _insert(
        isSingleton: true,
        replace: overrideAbstract,
        //?? (("$S" == "${dependency.runtimeType}") == false),
        name: tag,
        permanent: permanent,
        builder: builder ?? (() => dependency));
    return find<S>(tag: tag);
  }

  /// Create a new instance from builder class
  /// Example
  /// Get.create(() => Repl());
  /// Repl a = Get.find();
  /// Repl b = Get.find();
  /// print(a==b); (false)
  void create<S>(
    FcBuilderFunc<S> builder, {
    String name,
  }) {
    _insert(isSingleton: false, name: name, builder: builder);
  }

  void _insert<S>({
    bool isSingleton,
    String name,
    bool replace = true,
    bool permanent = false,
    FcBuilderFunc<S> builder,
  }) {
    assert(builder != null);
    String key = getKey(S, name);
    if (replace) {
      singl[key] = FcBuilder<S>(isSingleton, builder, permanent);
    } else {
      singl.putIfAbsent(
          key, () => FcBuilder<S>(isSingleton, builder, permanent));
    }
  }

  void removeDependencyByRoute(String routeName) async {
    List<String> keysToRemove = [];
    routesKey.forEach((key, value) {
      // if (value == routeName && value != null) {
      if (value == routeName) {
        keysToRemove.add(key);
      }
    });
    keysToRemove.forEach((element) async {
      await delete(key: element);
    });
    keysToRemove.forEach((element) {
      routesKey?.remove(element);
    });
    keysToRemove.clear();
  }

  bool isRouteDependecyNull<S>({String name}) {
    return (routesKey[getKey(S, name)] == null);
  }

  bool isDependencyInit<S>({String name}) {
    String key = getKey(S, name);
    return routesKey.containsKey(key);
  }

  void registerRouteInstance<S>({String tag}) {
    //  print("Register route [$S] as ${Get.currentRoute}");
    routesKey.putIfAbsent(getKey(S, tag), () => RouteObs.routing.current);
  }

  S findByType<S>(Type type, {String tag}) {
    String key = getKey(type, tag);
    return singl[key].getSependency();
  }

  void initController<S>({String tag}) {
    String key = getKey(S, tag);
    final i = singl[key].getSependency();

    if (i is DisposableInterface) {
      i.onInit();
      if (GetConfig.logEnable) print('[GET] $key has been initialized');
    }
  }

  /// Find a instance from required class
  S find<S>({String tag, FcBuilderFunc<S> instance}) {
    String key = getKey(S, tag);
    bool callInit = false;
    if (isRegistred<S>(tag: tag)) {
      if (!isDependencyInit<S>() &&
          GetConfig.smartManagement != SmartManagement.onlyBuilder) {
        registerRouteInstance<S>(tag: tag);
        callInit = true;
      }

      FcBuilder builder = singl[key];
      if (builder == null) {
        if (tag == null) {
          throw "class ${S.toString()} is not register";
        } else {
          throw "class ${S.toString()} with tag '$tag' is not register";
        }
      }
      if (callInit) {
        initController<S>(tag: tag);
      }

      return singl[key].getSependency();
    } else {
      if (!factory.containsKey(key))
        throw " $S not found. You need call Get.put<$S>($S()) before";

      if (GetConfig.logEnable)
        print('[GET] $S instance was created at that time');
      S _value = put<S>(factory[key].call() as S);

      if (!isDependencyInit<S>() &&
          GetConfig.smartManagement != SmartManagement.onlyBuilder) {
        registerRouteInstance<S>(tag: tag);
        callInit = true;
      }

      if (GetConfig.smartManagement != SmartManagement.keepFactory) {
        factory.remove(key);
      }

      if (callInit) {
        initController<S>(tag: tag);
      }
      return _value;
    }
  }

  /// Remove dependency of [S] on dependency abstraction. For concrete class use Get.delete
  void remove<S>({String tag}) {
    String key = getKey(S, tag);
    FcBuilder builder = singl[key];
    final i = builder.dependency;

    if (i is DisposableInterface) {
      i.onClose();
      if (GetConfig.logEnable) print('[GET] onClose of $key called');
    }
    if (builder != null) builder.dependency = null;
    if (singl.containsKey(key)) {
      print('error on remove $key');
    } else {
      if (GetConfig.logEnable) print('[GET] $key removed from memory');
    }
  }

  String getKey(Type type, String name) {
    return name == null ? type.toString() : type.toString() + name;
  }

  bool reset({bool clearFactory = true, bool clearRouteBindings = true}) {
    if (clearFactory) factory.clear();
    if (clearRouteBindings) routesKey.clear();
    singl.clear();
    return true;
  }

  /// Delete class instance on [S] and clean memory
  Future<bool> delete<S>({String tag, String key}) async {
    String newKey;
    if (key == null) {
      newKey = getKey(S, tag);
    } else {
      newKey = key;
    }

    if (!singl.containsKey(newKey)) {
      print('Instance $newKey not found');
      return false;
    }

    FcBuilder builder = singl[newKey];
    if (builder.permanent) {
      (key == null)
          ? print(
              '[GET] [$newKey] has been marked as permanent, SmartManagement is not authorized to delete it.')
          : print(
              '[GET] [$newKey] has been marked as permanent, SmartManagement is not authorized to delete it.');
      return false;
    }
    final i = builder.dependency;

    if (i is DisposableInterface) {
      await i.onClose();
      if (GetConfig.logEnable) print('[GET] onClose of $newKey called');
    }

    singl.removeWhere((oldkey, value) => (oldkey == newKey));
    if (singl.containsKey(newKey)) {
      print('[GET] error on remove object $newKey');
    } else {
      if (GetConfig.logEnable) print('[GET] $newKey deleted from memory');
    }
    // Get().routesKey?.remove(key);
    return true;
  }

  /// check if instance is registred
  bool isRegistred<S>({String tag}) => singl.containsKey(getKey(S, tag));

  bool isPrepared<S>({String tag}) => factory.containsKey(getKey(S, tag));
}
